<?php
namespace ID3\Blog;

require_once PLUGIN_DIR . 'core/class.post.php';


class Main extends \ID3\Bases\MainInstance{
    private $schedule_intervals = [];
    
    function __construct(){
        parent::__construct(__NAMESPACE__);

        // Add registration code here
        Post::register_hooks();
        add_action("id3/" . KEY . "/settings/save", [$this, 'save_handler'],5,1);
        
        $this->register_scheduled_update();
        

    }

    
    function save_handler($data){
        $this->updateSchedule($data->automatic_update_timeout,true);
    }
    

    function addScheduleInterval($key, $interval, $label){
        $this->schedule_intervals[$key] = [
            'interval' => $interval,
            'display' => __($label)
        ];
    }

    function register_schedule_interval( $schedules ){
        foreach ( $this->schedule_intervals as $key => $value) {
            $schedules[$key] = $value;
        }

        // $schedules['every_five_minutes'] = [
        //     'interval'  => 5 * MINUTE_IN_SECONDS,
        //     'display'   => __( 'Every 5 Minutes')
        // ];

        // $schedules['every_fifteen_minutes'] = [
        //     'interval'  => 15 * MINUTE_IN_SECONDS,
        //     'display'   => __( 'Every 15 Minutes')
        // ];

        // $schedules['every_thirty_minutes'] = [
        //     'interval'  => 30 * MINUTE_IN_SECONDS,
        //     'display'   => __( 'Every 30 Minutes')
        // ];

        return $schedules;
    }

    function register_scheduled_update(){
        $eventName = KEY . '-schedule-update';

        $this->addScheduleInterval('every_five_minutes',5 * MINUTE_IN_SECONDS, 'Every 5 minutes');
        $this->addScheduleInterval('every_fifteen_minutes',15 * MINUTE_IN_SECONDS, 'Every 15 minutes');
        $this->addScheduleInterval('every_thirty_minutes',30 * MINUTE_IN_SECONDS, 'Every 30 minutes');

        add_filter( 'cron_schedules', [$this,'register_schedule_interval']);
        add_action($eventName, [$this,'update']);

        // Create schedule only if it doesn't already exists
        if (! wp_next_scheduled ( $eventName )) {
            
            wp_schedule_event(time(), 'hourly', $eventName);
        }
    }


    function updateSchedule($update_timeout, $delayed = false){
        $eventName = KEY . '-schedule-update';
        $nextOccurance = time();

        if (wp_next_scheduled( $eventName ) !== false) {
            wp_clear_scheduled_hook($eventName);
        }

        if($delayed == true && $this->schedule_intervals!=null && isset($this->schedule_intervals[$update_timeout])){
            $nextOccurance = $this->schedule_intervals[$update_timeout]['interval'];
        }

        
        wp_schedule_event($nextOccurance, $update_timeout, $eventName);
    }


    function update(){
        $syncInstance = \ID3\Blog\Synchroniser::getInstance();
        $this->log('Automatic update start');
        $result = $syncInstance->start(['check_pending' => true]);
        $this->log('update result:' . json_encode($result),false);

        return $result;
    }
    // Filters
}

new Main();