<?php
namespace ID3\Blog;

require_once PLUGIN_DIR . 'core/class.synchronizer.php';

/**
 * Plugin Rest API 
 * @package ID3\Blog
 * Use this class to add REST call ability to your plugin
 * For more infos, see: "Building your REST API"
 */
class Rest extends \ID3\Bases\Rest{

    function __construct(){
        parent::__construct(__NAMESPACE__);

        // Add rest route to be able to call them from the back/front-end via ajax
        $this->addRoute('/sync/state', [$this, 'sync__state']);

        $startRouteOptions = [];
        $this->newRouteParam($startRouteOptions, 'date', false, 'string', 'Earliest date of articles');
        $this->newRouteParam($startRouteOptions, 'method', false, 'string', 'incremental or full');
        $this->addRoutePOST('/sync/start', [$this, 'sync__start'], $startRouteOptions);
        
        $this->addRoutePOST('/sync/clear', [$this, 'sync__clear']);

        $this->addRoutePOST('/signal_update', [$this, 'signal_update']);

        $this->addRouteGET('/pending_update', [$this, 'has_pending_update']);

        $this->addRouteGET('/users', [$this, 'get_users']);
    }

    // GET: /wp-json/id3-rest/blog/sync/state
    function sync__state(){
        $syncInstance = \ID3\Blog\Synchroniser::getInstance();
        return $syncInstance->state;
    }

    // POST: /wp-json/id3-rest/blog/sync/start
    function sync__start($request){
        $limit_date     = $request->get_param('date');
        $method         = $request->get_param('method');
        
        
        $syncInstance = \ID3\Blog\Synchroniser::getInstance();
        
        //error_log('Synchronization started');

        $result = $syncInstance->start([
            'method' => $method, 
            'date'   => $limit_date
        ]);

        return $result;
    }

    // POST: /wp-json/id3-rest/blog/sync/clear
    function sync__clear(){
        
        $syncInstance = \ID3\Blog\Synchroniser::getInstance();
        return $syncInstance->clear();
    }

    // POST: /wp-json/id3-rest/blog/signal_update
    function signal_update(){
        $timestamp = time();
        set_transient(KEY . '-pending-update', $timestamp);

        return [
            'timestamp' => Date('Y-m-d H:i:s', $timestamp)
        ];
    }

    // POST: /wp-json/id3-rest/blog/pending_update
    function has_pending_update(){
        $timestamp = get_transient(KEY . '-pending-update');
        $next_cron = wp_next_scheduled('id3-blog-schedule-update');
        
        return [
            'signal' =>  $timestamp === false ? null :  $timestamp,
            'next_update' => $next_cron
        ];
    }

    // GET: /wp-json/id3-rest/blog/users
    function get_users(){
        $userList = \get_users();
        $result = [];
        foreach($userList as $userInfo){
            $result[] = [
                'id' => $userInfo->ID,
                'name' => $userInfo->data->display_name
            ];
        }
        return $result;
    }
}