<?php
namespace ID3\Blog;

/**
 * Plugin shortcodes handler
 * @package ID3\Blog
 * Use this class to add REST call ability to your plugin
 * For more infos, see: "Adding shortcode"
 */
class Shortcode extends \ID3\Bases\Shortcode{

    function __construct(){
        parent::__construct(__NAMESPACE__);

        // define the list of shortcodes
        $list = [
            'list' => 'blogList',
            'carousel' => 'blogCarousel'    
        ];

        $this->registerAll($list);
    }
    
    /**
     * Shortcode to display a blog list
     * Usage : [id3_blog_list]
     * 
     * @param string $name 
     */
    function blogList($atts, $content=null){
        global $id3Components;
        

        // extract will turn any value/key in atts in a $variable='value' form
        $attributes = shortcode_atts([
            'column_count' => 4,
            'min_column_size' => 250,
            'max_count' => 6,
            'list_style' => 'grid',
            'order_by' => 'date/desc',
            'aspect_ratio_width' => 4,
            'aspect_ratio_height' => 3,
            'alignment' => 'none',
            'show_picture' => 'true',
            'show_button' => 'true',
            'show_excerpt' => 'true',
            'show_pagination' => 'false',
            'show_categories' => 'false',
            'align' => 'center',
            'categories' => '',
            'hide_current' => 'true'
        ], $atts);

        extract(
            $attributes
        );



        $currentPage = (isset($_GET['id3-blog-page'])) ? ((int) $_GET['id3-blog-page']) - 1 : 0;
        if($show_pagination !== 'true') $currentPage = 0;
        $exclude  = null;
        if($hide_current == 'true'){
            global $post;
            if($post != null){
                $exclude = $post->ID;
            }
        }

        
        $query = $this->getArticles($max_count, $order_by, $categories, $currentPage,$exclude);
        

        $result = '';
        $classList = ['id3-blog-list','id3-style-' .$list_style, 'id3-block-align-' . $align];
        if($show_picture == 'true') $classList[] = 'id3-blog-show-picture';
        if($show_button == 'true') $classList[] = 'id3-blog-show-button';
        if($show_excerpt == 'true') $classList[] = 'id3-blog-show-excerpt';
        if($show_pagination == 'true') $classList[] = 'id3-blog-show-pagination';
        if($show_categories == 'true') $classList[] = 'id3-blog-show-categories';

        $styleList = [];
        foreach ($attributes as $key => $value) {
            $styleList[] = '--id3-' . str_replace('_','-',$key) . ':' . $value;
        }

        if($query->have_posts()){
            ob_start();
            
            echo('<div class="' . implode(' ', $classList) .'" style="' . implode(';', $styleList) . '">');
            echo('<div class="id3-blog-articles-container id3-list-container">');
            $index=0;

            while ( $query->have_posts() ) {
                $query->the_post();

                $itemClasses = ['id3-blog-item'];
                    
                
                if($list_style == 'mosaic' && (max(0, $index % (($column_count * 2) + 1)) == 0)) $itemClasses[] = 'id3-blog-large-tile';
                
                $image = apply_filters('id3-blog/post-image', get_the_post_thumbnail_url());

                echo('<a href="' . get_the_permalink() . '" class="' . implode(' ', $itemClasses) . '">');
                echo('<article>');
                    echo('<figure>');
                    echo('<img src="'. $image .'" />');
                    echo('</figure>');
                    echo('<div class="id3-blog-item-info">');
                        echo('<h1>' . get_the_title() . '</h1>');
                        $this->renderPostCategories(get_the_category());
                        echo('<div class="id3-blog-item-excerpt">' . get_the_excerpt() . '</div>');
                        echo('<div class="id3-blog-item-button-container">');
                        
                        $id3Components->Button(__('Read more', TDOM));
                        
                        echo('</div>');
                    echo('</div>');
                echo('</article>');
                echo('</a>');

                $index++;
            }
            echo('</div>');

            if($show_pagination == 'true'){
                $pageRangeStart = max(0, $currentPage - 2);
                $pageRangeEnd = min($pageRangeStart + 5, $query->max_num_pages);
                $pageRangeEndSkip = min($pageRangeEnd + 3, $query->max_num_pages);
                if($pageRangeEnd == $query->max_num_pages) $pageRangeStart = $pageRangeEnd - 5;

                echo('<div class="id3-blog-list-navigation">');
                    
                    echo('<a href="?id3-blog-page=1" class="id3-blog-navigation-item" ' . ($currentPage == 0 ? 'disabled' : '') .'><i class="fal fa-angle-double-left"></i></a>');
                    for($i = 0;$i < $query->max_num_pages; $i++){
                        if($i < $pageRangeStart) continue;
                        if($i >= $pageRangeEnd) {
                            echo('<a href="?id3-blog-page=' . $pageRangeEndSkip . '" class="id3-blog-navigation-item"><i class="fal fa-angle-right"></i></a>');   
                            break;
                        }

                        echo('<a href="?id3-blog-page=' . ($i + 1) . '" class="id3-blog-navigation-item ' . ($currentPage == $i ? 'id3-current' : '') . '">' . ($i + 1) . '</a>');   
                    }
                    echo('<a href="?id3-blog-page=' . $query->max_num_pages . '" class="id3-blog-navigation-item" ' . ($pageRangeEnd == $query->max_num_pages ? 'disabled' : '' ) . '><i class="fal fa-angle-double-right"></i></a>');
                echo('</div>');
            }

            echo('</div>');


            $result = ob_get_contents();
            ob_clean();
        }

        return $result;
    }


    function blogCarousel($atts, $content=null){
        global $id3Components;

        // extract will turn any value/key in atts in a $variable='value' form
        $attributes = shortcode_atts([
            'max_count' => 10,
            'order_by' => 'date/desc',
            'aspect_ratio_width' => 4,
            'aspect_ratio_height' => 3,
            'alignment' => 'none',
            'show_button' => 'true',
            'show_categories' => 'false',
            'auto_next_timeout' => 5,
            'align' => 'center',
            'categories' => '',
            'hide_current' => 'true'
        ], $atts);

        extract(
            $attributes
        );

        $exclude  = null;
        if($hide_current == 'true'){
            
        }

        $query = $this->getArticles($max_count, $order_by,$categories, 0 ,$exclude);

        $result = '';
        $classList = ['id3-blog-carousel','id3-block-align-' . $align];
        $styleList = [];
        $attrList = ['data-auto-next-timeout="' . $auto_next_timeout . '"'];

        if($show_categories == 'true') $classList[] = 'id3-blog-show-categories';

        foreach ($attributes as $key => $value) {
            $styleList[] = '--id3-' . str_replace('_','-',$key) . ':' . $value;
        }

        if($query->have_posts()){
            ob_start();
            
            echo('<div class="' . implode(' ', $classList) .'" style="' . implode(';', $styleList) . '"' . implode(' ',$attrList) . ' >');
            echo('<div class="id3-blog-carousel-viewport" id3-swiperight="previous()"  id3-swipeleft="next()">');
            echo('<div class="id3-blog-item-container">');
            $index=0;

            while ( $query->have_posts() ) {
                $query->the_post();

                $itemClasses = ['id3-blog-item'];
                    
                if($show_button == 'true') $itemClasses[] = 'id3-blog-show-button';
                $image = apply_filters('id3-blog/post-image', get_the_post_thumbnail_url());

                echo('<article class="' . implode(' ', $itemClasses) . '">');
                echo('<figure>');
                echo('<a href="' . get_the_permalink() . '"><img src="'. $image .'" /></a>');
                echo('</figure>');
                
                
                
                echo('<div class="id3-blog-item-info">');
                echo('<h1>' . get_the_title() . '</h1>');
                
                $this->renderPostCategories(get_the_category());

                echo('<div class="id3-blog-item-resume">' . get_the_excerpt() . '</div>');
                echo('<div class="id3-blog-item-button-container">');
                $id3Components->ActionButton(__('Read more', TDOM), ['href'=> get_the_permalink() ]);
                //echo('<a href="' . get_the_permalink() . '" class="id3-button id3-type-action">' . __('Read more') . '</a>');
                echo('</div>');
                echo('</div>');
                echo('</article>');
                $index++;
            }

            echo('</div>');
            echo('</div>');

            echo('<nav class="id3-blog-carousel-navigation">');
                echo('<div class="id3-blog-bullets">');
                for ($i=0; $i < $index ; $i++) { 
                    $bulletClasses = ['id3-bullet-item'];
                    if($i == 0) $bulletClasses[] = 'id3-current';
                    echo('<i class="' . implode(' ', $bulletClasses) . '"  id3-click="select(' . $i . ')"></i>');
                }
                echo('</div>');

                echo('<i class="id3-carousel-nav-item fal fa-angle-left" id3-click="previous()"></i>');
                echo('<i class="id3-carousel-nav-item fal fa-angle-right" id3-click="next()"></i>');
            echo('</nav>');


            echo('</div>');
            $result = ob_get_contents();
            ob_clean();
        }

        return $result;
    }

    private function getArticles(int $max=6, $orderBy='date/desc', $categories='', $page=0, $exclude=null){
        $orderBy = explode("/",$orderBy);
        if(!is_array($orderBy)) $orderBy = [$orderBy,$orderBy];
        $queryArgs = [
            'posts_per_page' => $max,
            'orderby' => $orderBy[0],
            'order' => $orderBy[1],
            'offset' => $max * $page
        ];

        if($exclude != null){
            $queryArgs['post__not_in'] = explode(',', $exclude );
        }

        if($categories != ''){
            $queryArgs['category__in'] = explode(',', $categories);
        }

        $query = new \WP_Query($queryArgs);

        return $query;
    }

    private function renderPostCategories($list){
        
        echo('<div class="id3-blog-item-cats">');
        foreach ( $list as $category ) {
            echo '<em class="cat-' . $category->term_id . '">' . $category->cat_name . '</em>';
        }
        echo('</div>');
    }

}