/**
 * 
 * Main controller file
 * 
 */
id3Plugin
.controller('blogCtrl', 
function(
    $scope, $rootScope, $q, $timeout,
    $id3UI, $id3Rest, $id3Settings, $id3Lists){

    // change setting values to match your needs. All settings change in form should trigger auto-save
    $scope.settings = {
        source: null,
        source_custom: '',
        category_subscriptions: null,
        category_exceptions: null,
        last_sync: null,
        //supported_languages: ['fr','en'],
        automatic_update_timeout: 'hourly',
        force_sync_timeout: 5,
        article_author: 1
    };
    $scope.isReady = false;
    $scope.categoryList = [];
    $scope.authorList = [];
    $scope.sourceCategoryCount = 0;
    
    $scope.init = function(){
        
        $rootScope.isReady({moduleName: 'blog'}).then(_ => {

            $id3Lists.register('blogSources', [
                {key: 'blog.source.immo', label: 'Real estate (blog.source.immo)'},
                {key: 'custom', label: 'Custom'}
            ]);

            

            $id3Settings.load($scope,'settings')
            .then(_ => {
                $id3Rest.get('blog/users').then($userList => {
                    $scope.authorList = $userList;
                });

                if($scope.settings.source == null){
                    if($id3Lists.get('blogSources').length > 2){
                        $id3UI.wizard('first-use');
                    }
                    else{
                        // TODO: Remove this to use the wizard
                        $scope.settings.source = $id3Lists.get('blogSources')[0].key
                        $scope.settings.category_subscriptions = {
                            fr:{
                                all: true,
                                list:[]
                            },
                            en: {
                                all: true,
                                list: []
                            }
                        }
                    }
                }

                console.log('Plugin ID-3 Blog is ready to do stuff with', $scope.settings);
                
            })
            .then(_ => {
                console.log('about to update categories');
                $scope.fetchSourceCategoryCount().then( $count =>{
                    $scope.sourceCategoryCount = $count;
                    $scope.isReady = true;
                })
                

                // $scope.updateCategories().then( _ => {
                //     $id3Lists.register('blogCategories', $scope.categoryList);
                //     $scope.isReady = true;
                // });
            })
            .then(_ => {
                $id3Rest.get('blog/pending_update').then($response => {
                    $scope.pendingUpdate = null;
                    if($response.signal != null){
                        $response.signal = moment($response.signal*1000).toDate();
                    }
                    if($response.next_update != null){
                        $response.next_update = moment($response.next_update*1000).toDate();
                    }

                    if($response.signal != null && $response.next_update != null){
                        if(moment($response.next_update).isBefore($scope.settings.last_sync)){
                            $response.signal = null;
                        }

                        $scope.pendingUpdate = $response;
                    }
                });
            });
        });

    }

    $scope.editCategoryFilter = function($type){
        //$item = angular.copy($scope.settings['category_' + $type]);

        // $id3UI.dialog('category-filter',{type: $type, model: $item, settings: $scope.settings}).then( $updatedItem => {
        //     $scope.settings['category_' + $type] = $updatedItem;
        // })
        $id3UI.dialog('category-filter',{settings: angular.copy($scope.settings)}).then( $updatedSettings => {

            //console.log('updatedSettings', $updatedSettings);
            
            $scope.settings.category_subscriptions  = $updatedSettings.category_subscriptions;
            $scope.settings.category_exceptions     = $updatedSettings.category_exceptions;
            
            //$scope.settings['category_' + $type] = $updatedItem;
        })
    }

    $scope.getSupportedLanguages = function(){

    }

    $scope.getFilterCount = function($list){
        const filter = $scope.settings['category_' + $list];
        if(filter == undefined) return 0;

        return Object.keys(filter).reduce( (t,k) => t+filter[k].list.length,0);
    }
    $scope.updateCategories = function(){
        return $q( ($resolve, $reject) => {
            $scope.fetchSourceCategories().then( 
                $categories => {
                    $scope.categoryList = $categories;
                    $resolve();
                },
                $error => {
                    console.log('Problem with fetchSourceCategories', $error);
                    $resolve();
                }
            )
        });
    }

    $scope.getSourceEntrypoint = function(){
        if( $scope.settings.source == null){
            console.log('Source is null');
            return null;
        }
        if( $scope.settings.source == 'custom' && id3IsNullOrEmpty($scope.settings.source_custom) ) {
            return null;
        }

        const sourcePath = $scope.settings.source == 'custom' ? $scope.settings.source_custom : 'https://' + $scope.settings.source;
        return sourcePath;
    }
    $scope.fetchSourceCategoryCount = function(){
        const sourcePath = $scope.getSourceEntrypoint();
        if(sourcePath == null){
            $q.resolve(0);
        }
        
        const lEndpoint = `${sourcePath}/wp-json/id3-dlp/`;
        return $q( ($resolve, $reject) => {
            $id3Rest.from(lEndpoint).get('categoryCount')
            .then( 
                $response => {
                    $resolve($response);
                },
                $error => {
                    $resolve(0);
                }
            )
            .catch( _ => {
                $resolve(0);
            });
        });
    }
    $scope.fetchSourceCategories = function(){
        const sourcePath = $scope.getSourceEntrypoint();
        if(sourcePath == null){
            $q.resolve(0);
        }

        const lEndpoint = `${sourcePath}/wp-json/id3-dlp/`;
        return $q( ($resolve, $reject) => {
            $id3Rest.from(lEndpoint).get('categories')
            .then( 
                $response => {
                    $resolve($response.map($e => {
                        return {
                            key: $e.term_id,
                            label: $e.name,
                            article_count: $e.count,
                            parent: $e.parent
                        }
                    }));
                },
                $error => {
                    $resolve([]);
                }
            );
        });
    }

    $scope.newCategoryException = function($item){
        return {
            active: true,
            type: 'exceptions',
            all:false,
            list: []
        }
    }

    
    $scope.startSync = function(){
        $id3UI.wizard('synchronisation',null, $scope.settings).then(_ => {
            $id3Settings.load($scope,'settings');
        });
    }

    $scope.resetSync = function(){
        $id3UI.confirm('This will remove all articles previously synced.','Are you sure you want to continue?').then(_ => { 
            $id3Rest.post('/blog/sync/clear').then( $response => {
                console.log('blogCtrl/resetSync@sync/clear', $response);
                $id3UI.alert('All articles removed');
            })
        });
    }
});


