const { __ } = wp.i18n;

import icons from './components/icons.js'

// Get components from from wp.blockEditor
const { InspectorControls, BlockControls,AlignmentToolbar } = wp.blockEditor;
// Get components from Components
const { 
  Panel,
  PanelBody,
  SelectControl,
  RangeControl,
  ToggleControl,
  ButtonGroup,
  Button,
  CheckboxControl
} = wp.components;

// Get useSelect from from wp.data
const { useSelect ,useDispatch} = wp.data;


// Base class for Blog block renderers
// used as a container for all shared methods
class BlogBlockRendererBase extends ID3BlockRenderer{
  
  // Get a list of dummy articles
  getList(){
    if(this.list != undefined) return this.list;
    const itemCategories = this.getFakeCategories();
    const variations = [
      {span: 35,excerpt: "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua."},
      {span: 39,excerpt: "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat."},
      {span: 34, excerpt: "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod."},
      {span: 41, excerpt: "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur."},
      {span: 36,excerpt: "Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam."},
    ];

    const result = Array.from(new Array(12)).map( ($e,$i) => {
      const seed = `lorem${$i}`;
      const variant = variations.random();
      console.log('getList', variant);
      return Object.assign({
        title: 'Article ' + ($i+1),
        categories: itemCategories.takeRandom(1,5),
        image: `https://picsum.photos/seed/${seed}/420/360`,
      },variant)
    });

    this.list = result;
    return result;
  }

  // Load and return the available categories 
  getAvailableCategories(){
    const categories = useSelect(select => select('core').getEntityRecords('taxonomy', 'category') );
    return categories || [];
  }

  getFakeCategories(){
    return 'Nullam Convallis Elit Aliquet Finibus Curabitur Consequat Commodo Consectetur'.split(' ');
  }

  // Render the sort panel with common controls 
  renderSortPanel({orderBy,categories,hideCurrent}){
    const arrCategories = categories.split(',')
          .filter(c => c!='')     // Filter empty
          .map(c => parseInt(c)); // map to int
          
    const availCategories = this.getAvailableCategories();
   
    availCategories.forEach( c => c.isChecked = arrCategories.includes(c.id));
    
    const toggleCategory = ( (cat) => {
      const index = arrCategories.indexOf(cat.id);
      if(index >= 0) arrCategories.splice(index,1);
      else arrCategories.push(cat.id);

      cat.isChecked = arrCategories.includes(cat.id);

      this.setAttributes({categories : arrCategories.filter(c => c != '').join(',')});
    });
    
    return (
      <Panel>
        <PanelBody title={__("Sorting and filtering", "id3")} initialOpen={ false }>
            
          <SelectControl
                    label={__("Order by")}
                    value={ orderBy }
                    onChange={ ($content) => this.setAttributes( {orderBy: $content})}
                    options={ [
                      {label: 'Random', value: 'rand/rand'},
                      {label: 'Newest to oldest', value: 'date/desc'},
                      {label: 'Oldest to newest', value: 'date/asc'},
                      {label: 'A → Z', value: 'title/asc'},
                      {label: 'Z → A', value: 'title/desc'}
                    ] }
                /> 
          
          <ToggleControl label="Hide current" 
              checked={hideCurrent} 
              onChange={ ( value ) => this.setAttributes( {hideCurrent:  value} ) } 
            /> 
          

          <h3>Categories</h3> 
          {availCategories.map(
              $cat => {
                return(<CheckboxControl
                  label={ $cat.name }
                  value={ $cat.id }
                  checked={ $cat.isChecked }
                  onChange={ () => toggleCategory($cat) }
                  />)
              }
            )}

          
          </PanelBody>
      </Panel>
    )
  }
}



//#region BlogList


export class BlogListEditor extends ID3BlockEditorBase {

    constructor(){
        super({
            shortcode_name: 'id3_blog_list'
        });
        
        this.title = __( 'Blog list', 'id3-blog' );
        this.description = __( 'Display a list of blog', 'id3-blog' );
        this.category = 'id3-blog';
        this.icon = icons.blogList;
        this.supports = {
          html: true,
          align: true,
        }
        this.attributes = {
          orderBy:{
            type: 'string',  
            default : 'date/desc',
          },
          columnCount:{
            type: 'int',  
            default : 4,
          },
          minColumnSize:{
            type: 'int',  
            default : 250,
          },
          maxCount:{
            type: 'int',  
            default : 8,
          },
          aspectRatioWidth:{
            type: 'int',  
            default : 4,
          },
          aspectRatioHeight:{
            type: 'int',  
            default : 3,
          },
          alignment: {
            type: 'string',
            default: 'none',
          },
          showPicture: {
            type: 'bool',
            default: true,
          },
          showCategories: {
            type: 'bool',
            default: false,
          },
          showButton: {
            type: 'bool',
            default: true,
          },
          showExcerpt: {
            type: 'bool',
            default: true,
          },
          showPagination: {
            type: 'bool',
            default: false,
          },
          listStyle: {
            type: 'string',
            default: 'grid'
          },
          align:{
            type:'string',
            default: 'center'
          },
          categories: {
            type: 'string',
            default: ''
          },
          hideCurrent: {
            type: 'bool',
            default: true
          }
        };

        
        this.addRenderer( new BlogListRenderer() );
        this.addRenderer( new BlogListToolbar(),true );
        this.addRenderer( new BlogListSidebar(),true );
    }
}



class BlogListRenderer extends BlogBlockRendererBase{


  content({maxCount, columnCount, showPicture, showButton,showCategories,aspectRatioWidth,aspectRatioHeight,listStyle,showExcerpt,showPagination}){
    console.log('BlogListRenderer/content', maxCount, columnCount);

    const listLength = maxCount == -1 ? Number(columnCount)*2 : Number(maxCount);
    const list = this.getList().filter( (e,i) => i<listLength);
    
    this.classList.push('id3-style-' + listStyle);

    if(showPicture) this.classList.push('id3-blog-show-picture');
    if(showButton) this.classList.push('id3-blog-show-button');
    if(showExcerpt) this.classList.push('id3-blog-show-excerpt');
    if(showPagination) this.classList.push('id3-blog-show-pagination');
    if(showCategories) this.classList.push('id3-blog-show-categories');

    const result =    
        <div className={this.containerClassList()} style={this.containerStyles()} >
            <div class="id3-blog-articles-container">
            <>
              {list.map(
                  (e,i) => {
                    const itemClasses = ['id3-blog-item'];
                    const itemStyles = {};
                    
                    if(listStyle == 'mosaic' && (Math.max(0, i % ((columnCount * 2) + 1)) == 0)) itemClasses.push('id3-blog-large-tile');
                    if(listStyle == 'masonry') itemStyles['grid-row-end'] = 'span ' + e.span.toString();
                    
                    return <div className={itemClasses.join(' ')} style={itemStyles} >
                      <article>
              
                        <figure>
                          <img src={e.image} />
                        </figure>
                        
                        
                        <div class="id3-blog-item-info">
                          <h1>{e.title}</h1>
                          <div class="id3-blog-item-cats">
                            {e.categories.map( 
                              (cat) => {
                                return <em>{cat}</em>
                              }
                            )}
                          </div>
                          <div class="id3-blog-item-excerpt">{e.excerpt}</div>
                          <div class="id3-blog-item-button-container">
                            <div class="id3-button id3-type-normal id3-style-hollow">{__('Read more')}</div>
                          </div>
                        </div>
                      </article>
                    </div>
                  }
              )}
            </>
            </div>
            <div class="id3-blog-list-navigation">
              <div class="id3-blog-navigation-item" disabled><i class="fal fa-angle-left"></i></div>
              <div class="id3-blog-navigation-item id3-current">1</div>
              <div class="id3-blog-navigation-item">2</div>
              <div class="id3-blog-navigation-item">3</div>
              <div class="id3-blog-navigation-item"><i class="fal fa-angle-right"></i></div>
              <div class="id3-blog-navigation-item"><i class="fal fa-angle-double-right"></i></div>
            </div>
        </div>
    return result;
  }

  
}

class BlogListToolbar extends BlogBlockRendererBase{

  content({alignment}){
    const onChangeAlignment = ( newAlignment ) => {
      this.setAttributes( {
          alignment: newAlignment === undefined ? 'left' : newAlignment,
      } );
    };

    return (
      <BlockControls>
        <AlignmentToolbar
              value={ alignment }
              onChange={ onChangeAlignment }
          />   
        
      </BlockControls>
    )
  }

}

class BlogListSidebar extends BlogBlockRendererBase{
 

  content(params){
    return (
    <InspectorControls>
        { this.renderDisplayPanel(params) }
        
        { this.renderSortPanel(params) }
    </InspectorControls>
    )
  }

  renderDisplayPanel({maxCount, columnCount,showPicture,showButton,aspectRatioWidth,aspectRatioHeight,minColumnSize,listStyle,showExcerpt,showCategories,showPagination}){
    const maxColumnCount =  (maxCount > 0 ? Number(maxCount) : 12);
    //this.preload();
    
    const fnChangeAspectRatio = ($content) => {
      const [width, height] = $content.split('/');
      this.setAttributes({aspectRatioWidth: width, aspectRatioHeight: height});
    }

    return ( 
    <Panel>
      <PanelBody title={__("Display Settings", "id3")} initialOpen={ false }>

        <SelectControl
                label={__("Grid style")}
                value={ listStyle }
                onChange={(value) => this.setAttributes({ listStyle: value })}
                options={ [
                  {label: 'List', value: 'list'},
                  {label: 'Grid', value: 'grid'},
                  {label: 'Mosaic', value: 'mosaic'},
                  {label: 'Masonry', value: 'masonry'},
                ] }
            /> 
        
        <RangeControl label="Articles"
            value={ maxCount }
            onChange={ ( value ) => this.setAttributes( {maxCount:  value} ) }
            min={ 1 }
            max={ 12 }
          />

        {!['list','masonry','grid'].includes(listStyle) && (
          <RangeControl label="Columns"
            value={ columnCount }
            onChange={ ( value ) => this.setAttributes( {columnCount:  value} ) }
            min={ 1 }
            max={ maxColumnCount }
          />
        )}

        {['masonry','grid'].includes(listStyle) && (
          <RangeControl label="Min column size (px)"
            value={ minColumnSize }
            onChange={ ( value ) => this.setAttributes( {minColumnSize:  value} ) }
            min={ 1 }
            max={ 640 }
          />
        )}

        <ToggleControl label="Show picture" 
            checked={showPicture} 
            onChange={ ( value ) => this.setAttributes( {showPicture:  value} ) } 
          />    


        {showPicture && (
        <SelectControl
                label={__("Aspect ratio")}
                value={ [aspectRatioWidth,aspectRatioHeight].join('/') }
                onChange={fnChangeAspectRatio}
                options={ [
                  {label: '1 / 1', value: '1/1'},
                  {label: '4 / 3', value: '4/3'},
                  {label: '16 / 9', value: '16/9'},
                ] }
            />  
        )}

        <ToggleControl label="Show excerpt" 
            checked={showExcerpt} 
            onChange={ ( value ) => this.setAttributes( {showExcerpt:  value} ) } 
          />


        <ToggleControl label="Show categories" 
            checked={showCategories} 
            onChange={ ( value ) => this.setAttributes( {showCategories:  value} ) } 
          />

        <ToggleControl label="Show button" 
            checked={showButton} 
            onChange={ ( value ) => this.setAttributes( {showButton:  value} ) } 
          />

        <ToggleControl label="Show pagination" 
            checked={showPagination} 
            onChange={ ( value ) => this.setAttributes( {showPagination:  value} ) } 
          />
      </PanelBody>
    </Panel>
    )
  }


}

//#endregion

//#region BlogCarousel



export class BlogCarouselEditor extends ID3BlockEditorBase {

  constructor(){
      super({
          shortcode_name: 'id3_blog_carousel'
      });
      
      this.title = __( 'Blog carousel', 'id3-blog' );
      this.description = __( 'Display a list of article in a carousel', 'id3-blog' );
      this.category = 'id3-blog';
      this.icon = icons.blogCarousel;
      this.supports = {
        html: true,
        align: true
      }
      this.attributes = {
        alignment: {
          type: 'string',
          default: 'none',
        },
        orderBy:{
          type: 'string',  
          default : 'date/desc',
        },
        columnCount:{
          type: 'int',  
          default : 4,
        },
        maxCount:{
          type: 'int',  
          default : 10,
        },
        showButton: {
          type: 'bool',
          default: true,
        },
        showBullets: {
          type: 'bool',
          default: true,
        },
        showCategories: {
          type: 'bool',
          default: false,
        },
        autoNextTimeout: {
          type: 'int',
          default: 5,
        },
        aspectRatioWidth:{
          type: 'int',  
          default : 4,
        },
        aspectRatioHeight:{
          type: 'int',  
          default : 3,
        },
        align:{
          type:'string',
          default: 'center'
        },
        categories:{
          type: 'string',
          default: ''
        },
        exceptions: {
          type: 'string',
          default: ''
        }
      };

      
      this.addRenderer( new BlogCarouselRenderer() );
      this.addRenderer( new BlogCarouselSidebar(),true );
      this.addRenderer( new BlogCarouselToolbar(),true );
  }
}



class BlogCarouselRenderer extends BlogBlockRendererBase{

  content({maxCount, showButton, showBullets, showCategories}){
    
    const listLength = Number(maxCount);
    
    const list = this.getList();

    let bullets = [];
    if(showBullets == true){
      bullets = Array.from(new Array(listLength)).map(($e,$i) => $i);
      this.classList.push('id3-show-bullets');
    }
    if(showCategories == true) this.classList.push('id3-blog-show-categories');
    
    const result =    
        <div className={this.containerClassList()} style={this.containerStyles()} >
          <div class="id3-blog-carousel-viewport">
            <div class="id3-blog-item-container">
              {list.map(
                  (e) => {
                    const itemClasses = ['id3-blog-item'
                    //  ,'id3-color-secondary','id3-color-apply-bg'
                    ];
                    if(showButton) itemClasses.push('id3-blog-show-button');

                    return <article className={itemClasses.join(' ')}>
                      <figure><img src={e.image} /></figure>
                      
                      <div class="id3-blog-item-cats">
                      {e.categories.map( 
                        (cat) => {
                          return <em>{cat}</em>
                        }
                      )}
                      </div>
                        
                      <div class="id3-blog-item-info">
                        <h1>{e.title}</h1>
                        <div class="id3-blog-item-resume">{e.exerpt}</div>
                        <div class="id3-blog-item-button-container">
                          <div class="id3-button id3-type-action id3-style-fill">{__('Read more')}</div>
                        </div>
                      </div>
                    </article>
                  }
              )}
            </div>
          </div>
          <nav class="id3-blog-carousel-navigation">
            <div class="id3-blog-bullets">
              {
                bullets.map( $i => {
                  const bulletClass = ['id3-bullet-item'];
                  if($i == 0) bulletClass.push('id3-current');

                  return <i className={bulletClass.join(' ')}></i>
                })
              }
            </div>
            <i class="id3-carousel-nav-item fal fa-angle-left"></i>
            <i class="id3-carousel-nav-item fal fa-angle-right"></i>
          </nav>
        </div>
    return result;
  }
}


class BlogCarouselToolbar extends BlogBlockRendererBase{

  content({alignment}){
    const onChangeAlignment = ( newAlignment ) => {
      this.setAttributes( {
          alignment: newAlignment === undefined ? 'left' : newAlignment,
      } );
    };

    return (
      <BlockControls>
        <AlignmentToolbar
              value={ alignment }
              onChange={ onChangeAlignment }
          />   
      </BlockControls>
    )
  }
}

class BlogCarouselSidebar extends BlogBlockRendererBase{

  content(params){
    return (
      <InspectorControls>
          { this.renderDisplayPanel(params)}

          { this.renderSortPanel(params)}
      </InspectorControls>
    )
  }

  renderDisplayPanel({showBullets,autoNextTimeout, maxCount,showCategories,showButton,aspectRatioWidth,aspectRatioHeight}){
    const fnChangeAspectRatio = ($content) => {
      const [width, height] = $content.split('/');
      this.setAttributes({aspectRatioWidth: width, aspectRatioHeight: height});
    }
    
    return (
      <Panel>
          <PanelBody title={__("Display Settings", "id3")} initialOpen={ false }>
            <RangeControl label="Articles"
                value={ maxCount }
                onChange={ ( value ) => this.setAttributes( {maxCount:  value} ) }
                min={ 1 }
                max={ 10 }
              />
            <ToggleControl label="Show bullets" 
                checked={showBullets} 
                onChange={ ( value ) => this.setAttributes( {showBullets:  value} ) } 
              />

            <ToggleControl label="Show categories" 
                checked={showCategories} 
                onChange={ ( value ) => this.setAttributes( {showCategories:  value} ) } 
              />
              
            <ToggleControl label="Show button" 
                checked={showButton} 
                onChange={ ( value ) => this.setAttributes( {showButton:  value} ) } 
              />
            <SelectControl
                    label={__("Aspect ratio")}
                    value={ [aspectRatioWidth,aspectRatioHeight].join('/') }
                    onChange={fnChangeAspectRatio}
                    options={ [
                      {label: '1 / 1', value: '1/1'},
                      {label: '4 / 3', value: '4/3'},
                      {label: '16 / 9', value: '16/9'},
                    ] }
                />  
            <RangeControl
                    label={__("Next item timeout (seconds) ")}
                    value={ autoNextTimeout }
                    onChange={(value) => this.setAttributes( {autoNextTimeout: value} )}
                    min={1}
                    max={10}
                />  
             
          </PanelBody>
      </Panel>
    );
  }
}


//#endregion
